/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * Klasse zur Darstellung von Linealen.
 */
public class RulerPane extends JPanel {

    private static class Ruler extends JComponent implements MouseMotionListener, MouseListener {

	public static final int HORIZONTAL = 0;
	public static final int VERTICAL = 1;

	private int startPoint;
	private int orientation = HORIZONTAL;
	private int delta = 0;
	private FontMetrics fm;

	public Ruler() {this(HORIZONTAL);}

	public Ruler(int orientation) {
	    
	    this.orientation = orientation;
	    setBackground(Color.white);
	    setDoubleBuffered(true);
	    setFont(new Font("SansSerif", Font.PLAIN, 10));
	    setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
	    addMouseListener(this);
	    addMouseMotionListener(this);
	}
	
	public Dimension getPreferredSize() {return new Dimension(18, 18);}

	public void paintComponent(Graphics g) {
	    
	    Dimension d = getSize();
	    
	    g.setColor(Color.white); g.fillRect(0, 0, d.width, d.height);
	    g.setColor(Color.black); 

	    if (fm == null) {
		fm = g.getFontMetrics();
		if (orientation == VERTICAL) 
		    setFont(getFont().deriveFont(Font.PLAIN,
						 AffineTransform.getRotateInstance(-Math.PI/2)));
	    }

	    switch (orientation) {
	    case HORIZONTAL: 
		for (int i = -delta % 10; i < d.width; i += 10) 
		    if ((i + delta) % 50 != 0)
			g.drawLine(i, d.height, i, d.height - 5);
		for (int i = -delta % 50; i < d.width; i += 50) {
		    g.drawLine(i, d.height, i, d.height - 3);
		    g.drawString("" + (i + delta), i - fm.stringWidth("" + (i + delta)) / 2, 
				 d.height - 5);
		    g.drawLine(i, 0, i, d.height - 5 - fm.getAscent());
		}
		break;
		
	    case VERTICAL:
		for (int i = -delta % 10; i < d.height; i += 10) 
		    if ((i + delta) % 50 != 0)
			g.drawLine(d.width, i, d.width - 5, i);
		for (int i = -delta % 50; i < d.height; i += 50) {
		    g.drawLine(d.width, i, d.width - 3, i);
		    g.drawString("" + (i + delta), d.width - 5, 
				 i + fm.stringWidth("" + (i + delta)) / 2);
		    g.drawLine(0, i, d.width - 5 - fm.getAscent(), i);
		}
		break;
	    }
	}
	    
	public void mousePressed(MouseEvent e) {
	    
	    startPoint = delta + ((orientation == HORIZONTAL) ? e.getX() : e.getY());
	}
	
	public void mouseDragged(MouseEvent e) {
	    
	    delta = startPoint - ((orientation == HORIZONTAL) ? e.getX() : e.getY());
	    repaint();
	}
	
	public void mouseClicked(MouseEvent e) {

	    if (e.getClickCount() == 2) {

		delta = 0; repaint();
	    }
	}

	public void mouseReleased(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) {}
    }

    private JPanel center = new JPanel(new BorderLayout());
    private JPanel northPanel = new JPanel(new GridLayout(1, 1, 0, 0));
    private Ruler north = new Ruler(Ruler.HORIZONTAL);
    private Ruler west = new Ruler(Ruler.VERTICAL);

    public RulerPane() {

	super.setLayout(new BorderLayout());
	super.add("North", northPanel);
	super.add("West", west);
	super.add("Center", center);
	northPanel.add(north);
	northPanel.setBorder(new EmptyBorder(0, 19, 0, 0));
    }

    public Component add(Component c) {return center.add(c);}
    public Component add(Component c, int i) {return center.add(c, i);}
    public Component add(String s, Component c) {return center.add(s, c);}
    public void add(Component c, Object o) {center.add(c, o);}
    public void add(Component c, Object o, int i) {center.add(c, o, i);}

    public void showRulers() {

	add("North", northPanel);
	add("West", west);
	revalidate();
	repaint();
    }

    public void hideRulers() {

	remove(northPanel);
	remove(west);
	revalidate();
	repaint();
    }
    
    public static void main(String[] args) {
	
	JFrame f = new JFrame("asdfasdf");
	Container p = f.getContentPane();
	p.add(new RulerPane());

	f.setSize(500, 500);
	f.setVisible(true);
    }
}
