/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;
import javax.swing.*;

/**
 * Zur Erzeugung von Fortschrittanzeigen mit Hilfe des ProgressHandlers:
 * Klassen, die den ProgressHandler-Mechanismus unterstuetzen, werfen in Methoden
 * mit umfangreichen Berechnungen eine genau spezifizierte Anzahl von "ProgressEvents"
 * aus, indem sie die Methoden step() der registrierten ProgressHandler aufrufen.
 * Mit der Methode setBounds muss zuvor der Bereich festgelegt werden,
 * in dem die Progressanzeige verlaeuft. 
 * Die Methode step zaehlt den Fortschritt um eins nach oben und zeichnet die Leiste neu.
 * Laeuft dabei der Wert aus dem zuvor festgelegten Bereich hinaus, wird die Progressbar
 * auf 0 gesetzt. Mit setStep kann eine Mindestschrittweite festgelegt werden, unterhalb
 * derer ein Neuzeichnen (z.B. aus Effizienzgruenden) unterdrueckt wird.
 */
public class ProgressHandler {

    private JProgressBar pgb;
    private int min;
    private int max;
    private int value;
    private int step = 0;

    private ProgressHandler() {}

    /**
     * Erzeugt einen neuen ProgressHandler
     */
    public ProgressHandler(JProgressBar pgb) {

	this.pgb = pgb;
    }

    /**
     * legt die Grenzen auf (min, max) fest und initialisiert die Leiste auf 
     * min.
     */
    public void setBounds(int min, int max) {

	max--; this.min = min; this.max = max;
	pgb.setMinimum(min);
	pgb.setMaximum(max);
	pgb.setValue(min);
	pgb.repaint();
	value = min;
    }

    public void setStep(int s) {step = s;}

    /**
     * Erhoeht den Wert der PGB um eins und zeichnet sie neu, falls seit
     * dem letzten neuzeichnen mehr als step Schritte vergangen sind.
     */
    public void step() {

	value++;
	if (value > max) {
	    pgb.setValue(min);
	    pgb.repaint();
	} else if (value > pgb.getValue() + step) {
	    pgb.setValue(value);
	    pgb.repaint();
	}
    }
}
