/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.filetype;

import javax.swing.JFrame;
import java.io.*;

import picture.*;
import picture.gui.*;

/**
 * Klasse zum Zusammenfassen der Informationen zum Speichern/Laden
 * eines Bildes.
 *
 * @version 1.0 (c) 2000
 * @author Stefan Rhrich
 * @author Manuel Kauers
 */
public abstract class PictureType {

    /**
     * package-Access-Konstrukor wegen Zugriffsschutz.
     */
    PictureType() {
    }

    /**
     * Liefert die Dateiendung zurck.
     *
     * @return Dateiendung.
     */
    public abstract String getExtension();

    /**
     * Beschreibung des Bildtyps.
     *
     * @return Kurzbeschreibung.
     */
    public abstract String getDescription();

    /**
     * Liefert einen Speichern-Optionendialog zurck (<tt>null</tt>,
     * falls es keinen gibt).
     *
     * @param owner Parentframe
     * @param pic zu berbeitendes Bild
     * @return Speichern-Optionendialog */
    public DialogPane getSaveOptionsPane(JFrame f, Picture p) throws PictureException {
	return null;
    }

    /**
     * Ldt ein Bild im entsprechenden Format.
     *
     * @param in Eingabestream
     * @return gelesenes Bild
     */
    public abstract Picture load(InputStream in) throws PictureException, IOException;

    /**
     * Speichert das bergebene Bild im entsprechenden Format ab.
     *
     * @param p Zu speicherndes Bild
     * @param out Ausgabestream
     */
    public void save(Picture p, OutputStream out) throws PictureException, IOException {
	save(p, null, out);
    }

    /**
     * Speichert das bergebene Bild im entsprechenden Format mit Optionen ab.
     *
     * @param p Zu speicherndes Bild
     * @param p Optionen
     * @param out Ausgabestream
     */
    public abstract void save(Picture p, PictureProperty pp, OutputStream out) throws PictureException, IOException;
}
