/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.filetype;

import java.util.*;
import java.io.*;

import picture.*;

/**
 * PictureType-Fabrik.
 *
 * @author Stefan Roehrich
 */
public class PictureTypeFactory {

    private static Vector ptvec = new Vector();

    /**
     * Registriert einen neuen PictureType.
     *
     * @param pt neuer PictureType
     */
    public static void registerPictureType(PictureType pt) {
	ptvec.add(pt);
    }

    /**
     * Liefert den Standard-PictureType zurck.
     *
     * @return Standard-PictureType, z.B. zum Abspeichern.
     */
    public static PictureType getDefaultPictureType() {
	return (PictureType)ptvec.get(0);
    }

    /**
     * Liefert alle registrierten PictureTypes
     *
     * @return alle registrierten PictureTypes
     */
    public static ListIterator getAllPictureTypes() {
	return ptvec.listIterator();
    }

    /**
     * Liefert eine PictureType-Instanz zur bergebenen Dateiendung.
     *
     * @param ext Dateiendung
     * @return PictureType-Instanz, die entsprechenden Dateityp verarbeiten kann
     * @exception PictureException
     */
    public static PictureType getPictureTypeByExtension(String ext) throws PictureException {
	for (Iterator ii = ptvec.iterator(); ii.hasNext();) {
	    PictureType pt = (PictureType)ii.next();
	    if(ext.equalsIgnoreCase(pt.getExtension()))
		return pt;
	}
	// sollte eigentlich nicht erreicht werden ...
	throw new PictureException("Gewnschtes Dateiformat nicht registriert!");
    }

    /**
     * Ldt ein Bild aus Datei.
     *
     * @param fn Dateiname
     * @return Neues Bild
     * @exception IOException
     * @exception PictureException
     */
    public static Picture loadPicture(String fn) throws IOException, PictureException {

	Picture p = loadPicture(new File(fn));
	p.setFileName(fn);
	return p;
    }

    /**
     * Ldt ein Bild aus Datei.
     *
     * @param f File-Objekt
     * @return Neues Bild
     * @exception IOException
     * @exception PictureException
     */
    public static Picture loadPicture(File f) throws PictureException, IOException {

	String ext = null;
	StringTokenizer st = new StringTokenizer(f.getName(), ".");

	if (st.hasMoreTokens()) st.nextToken(); // skip name
	if (st.hasMoreTokens()) ext = st.nextToken();

	Picture p = null;

	if (ext != null)
	    p = getPictureTypeByExtension(ext).load(new FileInputStream(f));

	if (p == null)
	    throw new PictureException("Unbekanntes Datenformat oder fehlerhafte Datei");

	return p;
    }
}
