/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.filter;
import picture.*;

/**
 * Repraesentiert eine horizontale oder vertikale Spiegelung
 */
public class Reflection extends DefaultTransformation {

    public static final int HORIZONTAL = 0;
    public static final int VERTICAL = 1;

    private int mode = 0;

    /**
     * Erzeugt ein horizontale Spiegelung
     */
    public Reflection() {}

    /**
     * Erzeugt eine Spiegelung.
     * @param mode: HORIZONTAL oder VERTICAL
     */
    public Reflection(int mode) {

	this.mode = HORIZONTAL;
	if (mode == VERTICAL) this.mode = VERTICAL;
    }

    public int getNumberOfProgressSteps(Picture p) {return 0;}

    public Layer of(ProgressHandler ph, Layer p) {
	
	int w = p.getWidth(); int h = p.getHeight();
	double[][] data = new double[w][h];

	switch (mode) {
	case HORIZONTAL: 
	    for (int i = 0; i < w; i++)
		for (int j = 0; j < h; j++)
		    data[i][j] = p.data[w - 1 - i][j];
	    break;
	case VERTICAL:
	    for (int i = 0; i < w; i++)
		for (int j = 0; j < h; j++)
		    data[i][j] = p.data[i][h - 1 - j];
	    break;
	}

	return createNewInstance(p, data);
    }
}
