/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.net.URL;

import picture.*;
import picture.filter.*;
import picture.filetype.*;

/**
 * Klasse mit diversen graphischen Hilfsroutinen.
 */
public class GUIHelp {

    /**
     * Zustzliche Ausgabe der Exceptions?
     */
    public static boolean debug = false;

    /**
     * Defaultowner fr Fehlerdialoganzeige.
     */
    public static JFrame main = null;

    /**
     * z.B. Zur Darstellung von Histogrammen
     */
    public static class Graph extends JComponent {

	private double [] data = new double[]{0};
	private Color color = Color.red;

	public Graph(double [] data) {

	    this.data = data; init();
	}

	public Graph(int [] data) {

	    this.data = new double[data.length];
	    for (int i = 0; i < data.length; i++)
		this.data[i] = data[i];
	    init();
	}

	private void init() {

	    setBorder(new BevelBorder(BevelBorder.LOWERED));
	    setDoubleBuffered(true);
	}

	public Dimension getPreferredSize() {return new Dimension(data.length, 50);}

	/**
	 * Bestimmt die Farbe des Graphen
	 */
	public void setColor(Color c) {color = c;}

	public void paintComponent(Graphics g) {

	    double min = Double.POSITIVE_INFINITY;
	    double max = Double.NEGATIVE_INFINITY;

	    for (int i = 0; i < data.length; i++) {
		if (data[i] < min) min = data[i];
		if (data[i] > max) max = data[i];
	    }

	    Dimension d = getSize();
	    int oben = 0; int unten = d.height;
	    int rechts = d.width; int links = 0;
	    
	    g.setColor(Color.white); g.fillRect(0, 0, rechts, unten);
	    g.setColor(color);

	    int yAlt = d.height - (int) (d.height * (data[0] - min) / (max - min));
	    int xAlt = 0;
	    for (int i = 1; i < data.length; i++) {
		int x = i * d.width / (data.length - 1);
		int y = d.height - (int) (d.height * (data[i] - min) / (max - min));
		g.drawLine(x, y, xAlt, yAlt);
		xAlt = x; yAlt = y;
	    }
	}
    }

    /**
     * Eine Mischung aus FileFilter und PictureType
     */
    private static class MyFileFilter extends javax.swing.filechooser.FileFilter {

	PictureType [] pt;
	String beschreibung;
	String [] erweiterungen;

	/**
	 * Bei Verwendung dieses Konstruktors liefert die Methode convert das 
	 * Ergebnis null.
	 */
	public MyFileFilter(String beschreibung, String erweiterung) {

	    this.beschreibung = beschreibung;
	    this.erweiterungen = new String[] {erweiterung};
	}

	public MyFileFilter(PictureType pt) {

	    beschreibung = pt.getDescription();
	    erweiterungen = new String[] {pt.getExtension()};
	    this.pt = new PictureType[] {pt};
	}

	/**
	 * "Alle Bilddateien"
	 */
	public MyFileFilter() {

	    beschreibung = "Alle Bilddateien";
	    Vector v = new Vector();
	    
	    ListIterator li = PictureTypeFactory.getAllPictureTypes(); 
	    while(li.hasNext())	li.next();
	    while (li.hasPrevious()) 
		v.addElement(li.previous());

	    erweiterungen = new String[v.size()];
	    for (int i = 0; i < v.size(); i++)
		erweiterungen[i] = ((PictureType) v.elementAt(i)).getExtension();
	}

	/**
	 * Wandelt das uebergebene Bild in ein Objekt des Typs um, der
	 * durch diesen FileFilter repraesentiert wird.
	 */
	public Picture convert(Picture p) {
	    p.setPictureType(pt[0]);

	    return p;
	}

	public boolean accept(File f) {

	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    boolean b = false;

	    if (name == null) return false;

	    for (int i = 0; i < erweiterungen.length; i++)
		b |= name.toLowerCase().endsWith("." + erweiterungen[i]);

	    return b;
	}

	public String getDescription() {

	    String b = (erweiterungen.length == 1) ? " (*." + erweiterungen[0] + ")" : "";
	    return beschreibung + b;
	}
    }

    /**
     * Vorschaubild-Anzeige im Oeffnen-Dialog.
     */
    private static class FilePreviewer extends JComponent implements PropertyChangeListener {

	ImageIcon thumbnail = null;
	File f = null;
      
	public FilePreviewer(JFileChooser fc) {
	    setPreferredSize(new Dimension(100, 100));
	    fc.addPropertyChangeListener(this);
	}
	
	public void loadImage() {
	    try {
		if(f == null || !f.isFile())
		    return;
		String name = f.getName();
		ImageIcon tmpIcon = null;

		if (name.toLowerCase().endsWith("gif")) {

		    tmpIcon = new ImageIcon(f.getPath());

		} else {

		    Picture pic = PictureTypeFactory.loadPicture(f.getPath());
		    Picture picprev = pic.getPreview();

		    Image img = null;
		    
		    if (picprev != null)
			img = picprev.getImage();
		    else
			img = pic.getImage();
		    
		    tmpIcon = new ImageIcon(img);
		}

		int isd = Image.SCALE_DEFAULT; // weil die Zeile zu kurz ist...

		if (tmpIcon.getIconWidth() > 90) 
		    thumbnail = new ImageIcon(tmpIcon.getImage().getScaledInstance(90, -1, isd));
		else 
		    thumbnail = tmpIcon;

	    } catch (Exception e) {
		// Hier steht kein e.printStackTrance, da dieser Fehler bewusst in Kauf genommen
		// wird (z.B. wenn eine Datei ausgewaehlt wird, die nicht geoeffnet werden kann,
		// etwa ein Verzeichnis). Da dies fast staendig geschieht, wuerde an dieser 
		// Stelle ein Stackausdruck nur den Bildschirm vollmuellen mit unnuetzer 
		// Information. Aus diesem Grund nur die nachstehende, kurze Information
		if (debug)
		    System.err.println("keine Bilddatei ...");
		thumbnail = null;
	    }
	}
	
	public void propertyChange(PropertyChangeEvent e) {

	    String prop = e.getPropertyName();
	    if(prop == JFileChooser.SELECTED_FILE_CHANGED_PROPERTY) {
		f = (File) e.getNewValue();
		if(isShowing()) {
		    loadImage();
		    repaint();
		}
	    }
	}
      
	public void paint(Graphics g) {
	    
	    if(thumbnail == null) 
		loadImage();
	    
	    if (thumbnail != null) {
		
		int x = getWidth() / 2 - thumbnail.getIconWidth() / 2;
		int y = getHeight() / 2 - thumbnail.getIconHeight() / 2;
		
		if (y < 0) y = 0;
		if (x < 5) x = 5;
		
		thumbnail.paintIcon(this, g, x, y);
	    }
	}
    }

    /**
     * Filter abspeichern
     */
    public static void saveFilter(Component parent, DefaultTransformation dt, File select) {
	JFileChooser chooser = new JFileChooser(".");
	MyFileFilter filter = new MyFileFilter("Filter", "jf");
	chooser.addChoosableFileFilter(filter);
	chooser.setSelectedFile(select);
	int retval = chooser.showSaveDialog(parent);
	File theFile = chooser.getSelectedFile();
	
	if (retval == 0 && theFile != null)
	    try {
		main.setCursor(new Cursor(Cursor.WAIT_CURSOR));	
		dt.saveTo(theFile);
	    } catch (Exception ex) {
		error(ex);
	    } finally {
		main.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	    }
    }

    /**
     * Filterdatei ffnen.
     */
    public static File openFilter(Component parent, String selected) {

	JFileChooser chooser = new JFileChooser(selected);
	MyFileFilter filter = new MyFileFilter("Filter", "jf");
	chooser.addChoosableFileFilter(filter);
	int retval = chooser.showOpenDialog(parent);
	if (retval == 0) 
	    return chooser.getSelectedFile();
	
	return null;
    }
    
    /**
     * Bild abspeichern. (Speichern unter)
     */
    public static void saveTo(Component parent, Picture p, File select) 
	throws PictureException {

	// Geaendert 20.08.00 von mk.

	JFileChooser chooser = new JFileChooser(".");

	// Alle registrieten Typen zum Speichern zulassen.
	PictureType pt;
	PictureType myPt = p.getPictureType();
	MyFileFilter filter;
	MyFileFilter myFilter = new MyFileFilter(myPt);
	ListIterator li = PictureTypeFactory.getAllPictureTypes(); 
	while(li.hasNext())
	    li.next();
	while (li.hasPrevious()) {
	    pt = (PictureType) li.previous();
	    filter = new MyFileFilter(pt);
	    if (pt.equals(myPt))
		myFilter = filter;
	    chooser.addChoosableFileFilter(filter);
	}

	chooser.setFileFilter(myFilter);
	chooser.setAcceptAllFileFilterUsed(false);

	if (select != null)
	    chooser.setSelectedFile(select);
	
	int retval = chooser.showSaveDialog(parent);
	if (retval == 0) {

	    filter = (MyFileFilter) chooser.getFileFilter();
	    JFrame f = (parent instanceof JFrame) ? (JFrame) parent : null;
	    PictureProperty pp = p;
	    
	    DialogPane sp = filter.pt[0].getSaveOptionsPane(f, p);

	    if (sp != null) {
		sp.show();
		if (sp instanceof PictureProperty)
		    pp = (PictureProperty) sp;
		if (sp.getClosingCode() != sp.OK)
		    return;
	    }

	    File theFile = chooser.getSelectedFile();
	    try {
		main.setCursor(new Cursor(Cursor.WAIT_CURSOR));	
		filter.convert(p).saveToFile(theFile.getPath(), pp);
	    } catch(Exception e) {
		error(e);
	    } finally {
		main.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	    }
	}
    }

    /**
     * Bilddatei ffnen.
     */
    public static File openFile(Component parent, String selected) {

	JFileChooser chooser = new JFileChooser(selected);
	MyFileFilter alle = new MyFileFilter();
	chooser.addChoosableFileFilter(alle);

	PictureType pt;	MyFileFilter filter;
	ListIterator li = PictureTypeFactory.getAllPictureTypes();
	while(li.hasNext())
	    li.next();
	while (li.hasPrevious()) {
	    pt = (PictureType)li.previous();
	    filter = new MyFileFilter(pt);
	    chooser.addChoosableFileFilter(filter);
	}
	chooser.setAcceptAllFileFilterUsed(false);
	chooser.setAccessory(new FilePreviewer(chooser));
	chooser.setFileFilter(alle);

	int retval = chooser.showOpenDialog(parent);

	if (retval == 0) 
	    return chooser.getSelectedFile();
	
	return null;
    }

    /**
     * Fehlerdialog. Es wird die Verwendung von error(Exception) empfohlen.
     */
    public static void error(Component c, String s) {
	Frame f = (c instanceof Frame) ? (Frame) c : main;
	JOptionPane.showMessageDialog(f, s, "Fehler", JOptionPane.WARNING_MESSAGE);
    }

    /**
     * Fehlerdialog aus Exception.
     */
    public static void error(Exception ex) {
	String s = ex.getMessage();
	if (debug) {

	    ex.printStackTrace();
	    s = ex.getClass().getName() + ":\n" + s;

	} else if (s == null || s.length() == 0)

	    s = "Allgemeiner Fehler";

	error(main, s);
    }

    /**
     * Fehlerdialog aus Exception und bersetzung.
     */
    public static void error(Exception ex, String s) {
	if(debug)
	    ex.printStackTrace();
	error(main, s);
    }

    /**
     * Fehlerdialog. Es wird die Verwendung von error(Exception) empfohlen.
     */
    public static void error(String s) {
	error(main, s);
    }

    /**
     * Gibt Informationen ueber ein Layer in einem Infodialog aus.
     */
    public static void info(Picture p) {

	// Gesamt-Bild-Informationen:
	JLabel dateiName = new JLabel(p.getFileName());
	JLabel dateiFormat = new JLabel(p.getPictureType().getDescription());
	JLabel presentation = new JLabel(p.getPresentation().getPresentationType().getName());
	JLabel hoehe = new JLabel("" + p.getHeight());
	JLabel breite = new JLabel("" + p.getWidth());
	JLabel ebenen = new JLabel("" + p.getNumberOfLayers());

	JPanel oben = new JPanel(new BorderLayout(10, 10));
	JPanel obenLinks = new JPanel(new GridLayout(0, 1));
	JPanel obenDaten = new JPanel(new GridLayout(0, 1));
	
	obenLinks.add(new JLabel("Dateiname:")); obenDaten.add(dateiName);
	obenLinks.add(new JLabel("Dateiformat:")); obenDaten.add(dateiFormat);
	obenLinks.add(new JLabel("Prsentation:")); obenDaten.add(presentation);
	obenLinks.add(new JLabel("Hhe:")); obenDaten.add(hoehe);
	obenLinks.add(new JLabel("Breite:")); obenDaten.add(breite);
	obenLinks.add(new JLabel("Ebenenanzahl:")); obenDaten.add(ebenen);

	oben.add("West", obenLinks);
	oben.add("Center", obenDaten);
	oben.add("East", new JPicture(p.getPreview()));

	JComboBox layers = new JComboBox();
	final CardLayout cl = new CardLayout();
	final JPanel unten = new JPanel(cl);

	for (int i = 0; i < p.getNumberOfLayers(); i++) {

	    String bez = "Ebene " + (i + 1);
	    Layer pic = p.getLayer(i);

	    double min = pic.getMinimum();
	    double max = pic.getMaximum();
	    double mit = 0;
	    
	    for (int x = 0; x < p.getWidth(); x++)
		for (int y = 0; y < p.getHeight(); y++)
		    mit += pic.getValue(x, y);

	    mit /= p.getWidth() * p.getHeight();

	    double str = 0;

	    for (int x = 0; x < p.getWidth(); x++)
		for (int y = 0; y < p.getHeight(); y++)
		    str += sqr(pic.getValue(x, y) - mit);

	    str = Math.sqrt(str / (p.getWidth() * p.getHeight()));

	    JPanel panel = new JPanel(new BorderLayout(10, 10));
	    JPanel panelLinks = new JPanel(new GridLayout(0, 1));
	    JPanel panelDaten = new JPanel(new GridLayout(0, 1));
	    JPanel panelRechts = new JPanel(new BorderLayout(10, 10));
	    
	    panelLinks.add(new JLabel("Minimum:")); panelDaten.add(new JLabel("" + min));
	    panelLinks.add(new JLabel("Maximum:")); panelDaten.add(new JLabel("" + max));
	    panelLinks.add(new JLabel("Mittelwert:")); panelDaten.add(new JLabel("" + mit));
	    panelLinks.add(new JLabel("Streuung:")); panelDaten.add(new JLabel("" + str));
	    panelLinks.add(new JPanel()); panelDaten.add(new JPanel());
	    panelLinks.add(new JPanel()); panelDaten.add(new JPanel());

	    panelRechts.add("North", 
		       new JPicture(new Picture(new Layer[]{pic}).getPreview()));

	    int[] histogram = pic.histogram(128);
	    double[] h = new double[histogram.length];
	    for (int j = 0; j < h.length; j++)
		h[j] = Math.log(histogram[j] + 1);

	    panelRechts.add("South", new Graph(h));

	    panel.add("West", panelLinks);
	    panel.add("Center", panelDaten);
	    panel.add("East", panelRechts);

	    unten.add(bez, panel);
	    layers.addItem(bez);
	}

	layers.addItemListener(new ItemListener() {

	    public void itemStateChanged(ItemEvent e) {cl.show(unten, (String)e.getItem());}
	});

	JPanel comboBoxPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
	comboBoxPanel.add(layers);

	JPanel alles = new JPanel(new BorderLayout(10, 10));
	alles.add("North", oben);
	alles.add("Center", comboBoxPanel);
	alles.add("South", unten);

	JOptionPane.showMessageDialog(main, alles, "Bildinformationen", 
				      JOptionPane.PLAIN_MESSAGE);
    }

    private static final double sqr(double d) {return d * d;}

    public static Border createBorder(String title) {

	return new TitledBorder(BorderFactory.createEtchedBorder(), title);
    }

    /**
     * Liefert die URL eines Bildes mit dem bergebenen Namen zurck.
     */
    public static final URL getImageURL(String s) {

	return ClassLoader.getSystemResource("images/" + s + ".gif");
    }
}
