/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;

import picture.*;
import picture.filter.*;
import picture.filetype.*;

/**
 * Dialogfenster, das beim Speicherungsversuch eines
 * PGM-Bildes als PGMPictureProperty-Objekt verwendet werden kann
 */
public class SavePane extends DialogPane implements PGMPictureProperty {

    private class EventListener implements ActionListener, ChangeListener {

	public void actionPerformed(ActionEvent e) {

	    String k = e.getActionCommand();

	    if (k.equals("OK")) {
		closingCode = OK;
		dispose();
	    } else if (k.equals("Abbrechen")) {
		closingCode = ABBORD;
		dispose();
	    } else if (e.getSource() == grauwerte) {

		try {
		    long gw = new Long(grauwerte.getText()).longValue();
		    int bits = (int) (Math.log(gw) / Math.log(2));
		    if (bits > 0 && bits <= 64)
			bittiefe.setValue(bits);
		} catch (Exception ign) {}
	    }
	}

	public void stateChanged(ChangeEvent e) {

	    Object s = e.getSource();

	    if (s == bittiefe) {
		String t = "" + (long) (Math.pow(2, bittiefe.getValue()));
		if (t.length() > 10) {
		    t = "ca. " + t.charAt(0) + "." + t.substring(1, 2) + "E" + t.length();
		}
		grauwerte.setText(t);
	    } 
	}

    }

    private JRadioButton raw = new JRadioButton("RAW");
    private JRadioButton ascii = new JRadioButton("ASCII");

    private JTextField grauwerte = new JTextField("256", 10);
    private JSlider bittiefe = new JSlider(JSlider.HORIZONTAL, 1, 63, 8);

    private JTextArea comment = new JTextArea(5, 20);

    private EventListener ev = new EventListener();

    /**
     * Erzeugt ein neues SavePane
     */
    public SavePane(Frame owner, Picture pic) throws PictureException {

	super(owner, "PGM-Datenformat");
	updater.kill();

	if (pic == null)
	    throw new PictureException("Das Bild ist ungltig.");

	Container c = getContentPane();

	JPanel center = new JPanel(new BorderLayout());
	JPanel formatPanel = new JPanel(new GridLayout(0, 1));
	JPanel grayScale = new JPanel(new BorderLayout());
	JPanel textPanel = new JPanel(new GridLayout(1, 1));
	JPanel buttons = new JPanel(new GridLayout(1, 0, 5, 5));
	
	JButton ok = new JButton("OK");
	JButton abbrechen = new JButton("Abbrechen");

	/*
	 *   Layout
	 */
	c.setLayout(new BorderLayout(5, 5));

	c.add("Center", center);
	c.add("South", buttons);

	center.add("South", textPanel);
	center.add("East", formatPanel);
	center.add("West", grayScale);

	// Unten: Kommentar
	textPanel.setBorder(new TitledBorder(new EtchedBorder(), "Kommentar"));
	textPanel.add(new JScrollPane(comment));

	// Links: Raw oder ascii?
	formatPanel.setBorder(new TitledBorder(new EtchedBorder(), "Format"));
	formatPanel.add(raw);
	formatPanel.add(ascii);

	// Rechts: Graustufenanzahl
	grayScale.setBorder(new TitledBorder(new EtchedBorder(), "Farbtiefe"));
	JPanel r = new JPanel(new GridLayout(0, 1));
	JPanel p = new JPanel();
	p.add(grauwerte); p.add(new JLabel(" Graustufen")); r.add(p);
	grayScale.add("Center", r);
	grayScale.add("South", bittiefe);
	bittiefe.setMinorTickSpacing(1);
	bittiefe.setMajorTickSpacing(62);	
	bittiefe.setSnapToTicks(true);
	bittiefe.setPaintLabels(true);
	bittiefe.setPaintTicks(false);
	Dictionary dic = bittiefe.getLabelTable();
	dic.put(new Integer(32), new JLabel(". . . Bits/Pixel . . ."));
	bittiefe.setLabelTable(dic);

	buttons.setBorder(new EmptyBorder(10, 10, 10, 10));
	buttons.add(ok);
	buttons.add(abbrechen);
	buttons.add(new JPanel());

	pack();
	Toolkit tk = Toolkit.getDefaultToolkit();
	Dimension screen = tk.getScreenSize();
	Dimension size = getSize();
	setLocation((screen.width - size.width) / 2, (screen.height - size.height) / 2);

	/*
	 *  Verkabelung
	 */
	ok.addActionListener(ev); ok.setActionCommand("OK");
	abbrechen.addActionListener(ev); abbrechen.setActionCommand("Abbrechen");

	ButtonGroup bg = new ButtonGroup();
	bg.add(raw); raw.setSelected(true);
	bg.add(ascii);

	bittiefe.addChangeListener(ev);
	grauwerte.addActionListener(ev);
    }

    /**
     * Gibt den maximalen Grauwert des Bildes an.
     * Der minimale Grauwert des Bildes ist stets 0, somit wird
     * ueber diese Methode die Anzahl der Graustufen und damit
     * insbesondere die Groesse (Datenmenge) des Bildes festgelegt.
     */
    public long getNumberOfGrayScales() {

	long l = 0;
	try {
	    l = new Long(grauwerte.getText()).longValue();
	    if (l <= 1)
		throw new Exception();
	} catch (Exception e) { 
	    // Falls im Textfeld keine Zahl angegeben ist (dann wirft der Longkonstruktor
	    // eine Exception) oder die angegebene Zahl negativ ist (dann werfe ich selbst
	    // eine Exception, s.o.) wird der Wert aus dem Schieberegler genommen. Dieser
	    // kann keine ungueltigen Daten enthalten.
	    l = (long) (Math.pow(2, bittiefe.getValue()));
	}
	return l - 1;
    }

    /**
     * Gibt an, ob die Datei im Raw- oder im 
     * Ascii-Format geschrieben werden soll. Zurueckgegeben
     * werden RAW_FORMAT oder ASCII_FORMAT.
     */
    public int getFormat() {

	if (raw.isSelected())
	    return RAW_FORMAT;

	return ASCII_FORMAT;
    }

    /**
     * Gibt etwaige Kommentare ab, die in der
     * Datei gespeichert werden sollen.
     */
    public String [] getComment() {

	StringTokenizer t = new StringTokenizer(comment.getText(), "\n");
	Vector v = new Vector();
	while (t.hasMoreTokens())
	    v.addElement(t.nextToken());

	String[] c = new String[v.size()];
	for (int i = 0; i < c.length; i++)
	    c[i] = (String) v.elementAt(i);
	return c;
    }

    /** gibt null zurueck*/
    public Picture getResultingPicture() {return null;}
}
