/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import picture.*;
import picture.filter.*;
import picture.math.Term;

/**
 * Diese Klasse erlaubt die Einstellungen zur Skalierung von Grauwerten in PGM-Bildern
 * und repraesentiert das zugehoerige Swing-Eingabe-Fenster
 */
public class ScalePane extends DialogPane {

    private static final int GENERISCH = 0;
    private static final int FORMEL = 1;

    /**
     * Mit dieser Klasse ist eine Veranschaulichung der Farbveraenderung moeglich
     * Sie beinhaltet gleichzeitig die Implementierungen der noetigen Eventlistener
     */
    private class ScalePreview extends JComponent implements ChangeListener, ActionListener {
	
	public double obergrenze = 1;
	public double untergrenze = 0;

	private final int grauzahl = 256;
	private final int hOben = 0;	    
	private final int hUnten = 50;  // Histogrammbereich
	private final int sOben = 60;	    
	private final int sUnten = 70; // Skalabereich
	private final int gOben = 0;	    
	private final int gUnten = 50; // Graphbereich

	private Color [] farben = new Color[grauzahl];
	private Complex [][] xComplex = new Complex[grauzahl][1];
	private String [] xString = new String[]{"x"};
	private int[] histogramm;
	private int[] logHistogramm;
	private int maxH;
	private int logMaxH;
	
	public ScalePreview() {
	    
	    histogramm = pic.histogram(grauzahl);
	    logHistogramm = new int[grauzahl];
	    
	    for (int i = 0; i < grauzahl; i++) {
		
		int k = i * 255 / grauzahl;
		farben[i] = new Color(k, k, k);
		xComplex[i][0] = new Complex(i * 1.0 / grauzahl, 0);
		
		if (maxH < histogramm[i])
		    maxH = histogramm[i];		
	    } 
	    
	    for (int i = 0; i < grauzahl; i++) {
		logHistogramm[i] = (int) Math.log(histogramm[i] + 1);
		if (logMaxH < logHistogramm[i])
		    logMaxH = logHistogramm[i];
	    }
	    
	    setDoubleBuffered(true);	    
	}

	public Dimension getPreferredSize() {
	    
	    return new Dimension(grauzahl, getMaxY());
	}

	private int getMaxY() {

	    int hoehe = Math.max(hUnten, sUnten);
	    return Math.max(hoehe, gUnten);
	}

	public void stateChanged(ChangeEvent e) {

	    if (e.getSource() == unten) {
		setzeUntergrenze(unten.getValue() / 100.0);
		oben.setValue((int) (obergrenze * 100));
		oben.repaint();
	    } else if (e.getSource() == oben) {
		setzeObergrenze(oben.getValue() / 100.0);
		unten.setValue((int) (untergrenze * 100));
		unten.repaint();
	    } else if (e.getSource() == formelCB) {
		boolean b = formelCB.isSelected();
		oben.setEnabled(!b);
		unten.setEnabled(!b);
		formel.setEnabled(b);
		skalierungsart = b ? FORMEL : GENERISCH;
	    }

	    if (unten.getValue() == oben.getMaximum())
		unten.setValue(unten.getValue() - 1);
	    else if (oben.getValue() == unten.getValue())
		oben.setValue(oben.getValue() + 1);

	    vorschauAktualisieren();
	}

	private void vorschauAktualisieren() {

	    try {
		switch (skalierungsart) {
		case GENERISCH: 
		    scale.setBounds(unten.getValue() / 100.0, oben.getValue() / 100.0); break;
		case FORMEL: scale.setScaleFunction(new Term(formel.getText()), 256); break;
		}
	    } catch (IllegalArgumentException e) {
		GUIHelp.error(e, "Syntaxfehler");
	    } catch (ArithmeticException e) {
		GUIHelp.error(e);
	    }

	    updater.add(scale);
	    repaint();
	}

	public void actionPerformed(ActionEvent e) {

	    String k = e.getActionCommand();
	    if (k.equals("Abbrechen")) {
		closingCode = ABBORD;
		dispose();
	    } else if (k.equals("OK")) {
		closingCode = OK;
		dispose();
	    } else if (e.getSource() == formel) 
		vorschauAktualisieren();	    
	}

	public void setzeUntergrenze(double d) {

	    if (d < 0) d = 0;
	    if (d > 1) d = 1;
	    
	    untergrenze = d;
	    if (obergrenze < untergrenze)
		obergrenze = untergrenze;

	    repaint();
	}

	public void setzeObergrenze(double d) {

	    if (d < 0) d = 0;
	    if (d > 1) d = 1;

	    obergrenze = d;
	    if (untergrenze > obergrenze)
		untergrenze = obergrenze;

	    repaint();
	}

	public void paint(Graphics g) {

	    Dimension d = getSize(); d.height--;
	    double step = d.width * 1.0 / grauzahl;

	    int maxY = getMaxY();

	    int hOben = this.hOben * d.height / maxY;
	    int hUnten = this.hUnten * d.height / maxY;
	    int sOben = this.sOben * d.height / maxY;
	    int sUnten = this.sUnten * d.height / maxY;
	    int gOben = this.gOben * d.height / maxY;
	    int gUnten = this.gUnten * d.height / maxY;

	    int yHAlt = Integer.MAX_VALUE;
	    int yGAlt = Integer.MAX_VALUE;
	    Term t = null;

	    try {
		t = new Term(formel.getText());
	    } catch (Exception e) {}

	    // Bestimme Maximum und Minimum der Skalierungsfunktion auf [0, 1]
	    double minFunktion = Double.POSITIVE_INFINITY;
	    double maxFunktion = Double.NEGATIVE_INFINITY;
	    double wert = 0;
	
	    if (skalierungsart == FORMEL) 
		try {
		    for (int i = 0; i < grauzahl; i++) {
			//z[0] = xComplex[i];
			//z[0] = t.einsetzen(xG, z);
			wert = t.einsetzen(xString, xComplex[i]).getRealPart();
			//wert = z[0].getRealPart();
			if (wert > maxFunktion) maxFunktion = wert;
			if (wert < minFunktion) minFunktion = wert;
		    }
		} catch (Exception ign) {}
	    
	    for (int i = 0; i < grauzahl; i++) {

		int x = (int) (i * step);

		// Darstellung des Histogramms
		g.setColor(Color.red);
		int yH = logarithmisch.isSelected() ? logHistogramm[i] : histogramm[i];
		yH = hUnten + (hOben - hUnten) * yH / 
		    (logarithmisch.isSelected() ? logMaxH : maxH);
		if (yHAlt != Integer.MAX_VALUE)
		    g.drawLine(x - (int) step, yHAlt, x, yH);
		yHAlt = yH;
		
		// Darstellung der Skala
		g.setColor(farben[i]);
		g.fillRect(x, sOben, (int) step + 1, sUnten - sOben);

		// Darstellung des Graphen
		switch (skalierungsart) {
		case FORMEL: 
		    try {
			//z[0] = new Complex(i * 1.0 / grauzahl, 0);
			wert = t.einsetzen(xString, xComplex[i]).getRealPart();
		    } catch (Exception e) {
			yGAlt = Integer.MAX_VALUE;
			wert = -1E20; // => yG = Integer.MAX_VALUE
		    }
		    wert = (wert - minFunktion) / (maxFunktion - minFunktion);
		    break;
		case GENERISCH:
		    wert = (i - grauzahl * untergrenze) / (obergrenze - untergrenze);
		    wert /= grauzahl;
		    break;
		}
		if (wert > 1) wert = 1;
		if (wert < 0) wert = 0;
		int yG = gUnten + (int) ((gOben - gUnten) * wert);
		g.setColor(Color.green);
		if (yGAlt != Integer.MAX_VALUE)
		    g.drawLine(x - (int) step, yGAlt, x, yG);
		yGAlt = yG;
	    }
	}
    }
    
    private int skalierungsart = GENERISCH;
    
    private Picture pic;
    private GrayScale scale = new GrayScale(0, 1);

    private JSlider unten = new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
    private JSlider oben = new JSlider(JSlider.HORIZONTAL, 0, 100, 100);

    private JCheckBox logarithmisch = new JCheckBox("logarithmisches Histogramm");
    private JCheckBox formelCB = new JCheckBox("frei:");
    private JTextField formel = new JTextField("x");

    /**
     * Erzeugt ein neues ScalePane
     */
    public ScalePane(Frame owner, Picture pic) throws PictureException {

	this(owner, pic, new GrayScale(0, 1));
    } 

    public ScalePane(Frame owner, Picture pic, GrayScale gs) throws PictureException {

	super(owner, "Grauwertskalierung", pic);

	this.pic = pic;

	Picture preview = pic.getPreview();
	JPicture jpreview = new JPicture(preview);

	ScalePreview prev = new ScalePreview();

	/*
	 *   Erzeugung der Komponenten
	 */

	Container mitte = getContentPane();

	JPanel werkzeuge = new JPanel(new GridLayout(0, 1));
	JPanel slider1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
	JPanel slider2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
	JPanel formelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));

	JPanel logarithmischPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));

	/*
	 *   Layout
	 */

	slider1.add(this.oben);  slider1.add(new JLabel(" Obergrenze"));
	slider2.add(this.unten); slider2.add(new JLabel(" Untergrenze"));
	formelPanel.add(formelCB); 
	formelPanel.add(formel);

	logarithmischPanel.add(logarithmisch);

	werkzeuge.add(slider1);
	werkzeuge.add(slider2);
	werkzeuge.add(formelPanel);
	werkzeuge.add(logarithmischPanel);
	
	mitte.add("North", prev);
	mitte.add("Center", werkzeuge);

	prepareWindow();

	/*
	 * Initialisierung
	 */

	double [] bounds = gs.getBounds();
	unten.setValue((int) (100 * bounds[0]));
	oben.setValue((int) (100 * bounds[1]));

	Term t = gs.getScaleFunction();
	if (t != null) 
	    formel.setText(t.toString());

	/*
	 *  Verkabelung
	 */

	this.unten.addChangeListener(prev);
	this.oben.addChangeListener(prev);

	logarithmisch.addChangeListener(prev);
	logarithmisch.setSelected(true);
	formelCB.addChangeListener(prev);
	formel.addActionListener(prev);
	formel.setFont(new Font("Monospaced", Font.PLAIN, 14));
	formel.setEnabled(false);
	formel.setColumns(20);

	if (gs.getMode() == gs.TERM) {
	    oben.setEnabled(false);
	    unten.setEnabled(false);
	    formel.setEnabled(false);
	    formelCB.setSelected(true);
	}
    }

    /** Gibt die Transformation aus den aktuellen Einstellungen zurueck */
    public Transformation getTransformation() {return scale;}

    /**
     * Gibt das Pictureobjekt zurueck, das entsteht, wenn man die 
     * Filterung anhand der Benutzereinstellungen durchfuehrt.
     * Im Fall, dass bei der Filterung ein Fehler auftritt, wird 
     * die Identitaet zurueckgegeben.
     */
    public Picture getResultingPicture() {

	pic.getProgressHandler().setBounds(0, pic.getWidth());
	try {
	    return getTransformation().of(pic);
	} catch (Exception e) {}
	return pic;
    }
}
