/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.io;

import java.util.*;
import java.io.*;

/**
 * Input stream which transforms its data via
 * Differential-Decoding.
 *
 * @version 0.1
 * @author Stefan Rhrich */
public class DiffInputStream extends FilterInputStream {

    private int lastchar;

    /**
     * Initialize and set input stream. */
    public DiffInputStream(InputStream in) {
	super(in);
    }

    /**
     * Reads and transforms a byte from the underlying input
     * stream.
     *
     * @return tranformed byte or -1 for end of file
     * @exception IOException
     */
    public int read() throws IOException {
        int t = in.read();
	if(t == -1)
	    return -1;
	else
	    return lastchar = (lastchar+t) & 0xff;
    }

    /**
     * Reads and transforms a byte array from the underlying input
     * stream. The data is read byte for byte via read() from the
     * underlying input stream.
     *
     * @param b where to write the bytes to
     * @param off starting offset
     * @param len how many bytes to read
     * @return number of bytes bytes actually read
     * @exception IOException */
    public int read(byte[] b, int off, int len) throws IOException {
	int t, i;
        for(i = off; i < off+len; i++)
            if((t = read()) == -1)
		break;
	    else
		b[i] = (byte)t;
	return i;
    }

    /**
     * Do nothing.
     */
    public void mark(int readlimit) {
    }

    /**
     * Do nothing.
     */
    public void reset() throws IOException {
    }

    /**
     * @return false
     */
    public boolean markSupported() {
	return false;
    }

    /**
     * Skips bytes.
     *
     * @param n how many bytes to skip.
     * @return number of bytes actually skipped
     * @exception IOException */
    public long skip(long n) throws IOException {
	long i;
	int t;
	for(i = 0; i < n; i++)
	    if((t = read()) == -1)
		break;
	return i;
    }

    public static void main(String[] args) throws IOException {
	InputStream a = new DiffInputStream(System.in);
        int t;
        while((t = a.read()) != -1)
            System.out.write(t);
        System.out.close();
    }
}
