/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.io;

import java.util.*;
import java.io.*;

/**
 * Output stream which writes the length of its data before the data
 * itself after a flush().
 *
 * @version 0.1
 * @author Stefan Rhrich */
public class KnownLengthOutputStream extends FilterOutputStream {

    private ByteArrayOutputStream buf = new ByteArrayOutputStream();
    /**
     * Initialize and set output stream. */
    public KnownLengthOutputStream(OutputStream out) {
        super(out);
    }

    /**
     * Write byte.
     *
     * @param b byte to write
     * @exception IOException */
    public void write(int b) throws IOException {
	buf.write(b);
    }

    /**
     * Write byte array.
     *
     * @param b byte array to write
     * @param off starting offset
     * @param len number of bytes to write
     * @exception IOException */
    public void write(byte[] b, int off, int len) throws IOException {
	buf.write(b, off, len);
    }

    /**
     * Transforms the cached data and writes it to the underlying
     * output stream, which is flushed, too.
     *
     * @exception IOException. */
    public void flush() throws IOException {
	int len = buf.size();
	if(len > 0) {
	    out.write(len >> 24);
	    out.write(len >> 16);
	    out.write(len >> 8);
	    out.write(len);
	    buf.writeTo(out);
	    buf.reset();
	}
	out.flush();
    }

    /**
     * Closes the underlying output stream.
     *
     * @exception IOException. */
    public void close() throws IOException {
	int len = buf.size();
	if(len > 0) {
	    out.write(len >> 24);
	    out.write(len >> 16);
	    out.write(len >> 8);
	    out.write(len);
	    buf.writeTo(out);
	    buf.reset();
	}
	out.close();
    }

    public static void main(String[] args) throws IOException {
	OutputStream a = new KnownLengthOutputStream(System.out);
        int t;
        while((t = System.in.read()) != -1)
            a.write(t);
        a.close();
    }
}
