/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.io;

import java.util.*;
import java.io.*;

/**
 * Output stream which transforms its data via
 * Move-To-Front-Encoding.
 *
 * @version 0.1
 * @author Stefan Rhrich */
public class MTFOutputStream extends FilterOutputStream {

    private int[] index = new int[256];

    /**
     * Initialize and set output stream. */
    public MTFOutputStream(OutputStream out) {
        super(out);
	for(int i=0; i < 256; i++)
	    index[i] = i;
    }

    /**
     * Encode and write byte.
     *
     * @param b byte to write
     * @exception IOException */
    public void write(int b) throws IOException {
	int t;
	out.write(t = index[b]);
	for(int i=0; i < 256; i++)
	    if(index[i] < t)
		index[i]++;
	index[b] = 0;
    }

    /**
     * Encode and write byte array. The data is written byte for byte
     * with write(int) to the underlying output stream.
     *
     * @param b byte array to write
     * @param off starting offset
     * @param len number of bytes to write
     * @exception IOException */
    public void write(byte[] b, int off, int len) throws IOException {
	for(int i = off; i < off+len; i++)
	    write(ByteUtils.unsignedByte2Int(b[i]));
    }

    /**
     * Closes the underlying output stream.
     *
     * @exception IOException. */
    public void close() throws IOException {
	out.close();
    }

    public static void main(String[] args) throws IOException {
	OutputStream mtf = new MTFOutputStream(System.out);
        int t;
        while((t = System.in.read()) != -1)
            mtf.write(t);
        mtf.close();
    }
}
